/*
  (C) 2016 Leonid Eremin and Aleksey Eremin
  Arduino Pro Mini 
  http://ru.aliexpress.com/item/Free-Shipping-new-version-2pcs-lot-Pro-Mini-328-Mini-ATMEGA328-5V-16MHz-for-Arduino/1731218090.html


  Чтение из часов реального времени RTC ZS-043 ( DS1307 I2C) времени и отопбражение на LCD
  а при вводе через Serial настраивается точное время RTC
  Строка вида: YYYY:MM:DD:hh:mm:ss@
  
  каждые 30 минут звучит пищалка
  каждую секунду меняется яркость (ШИМ) и цвет RGB-светодиода
  А еще измеряем температуру и отображаем в первой строке через 15 сек
  
  
// Date and time functions using a DS1307 RTC connected via I2C and Wire lib
// RTC - Arduino
// -------------
// GND - GND 
// VCC - 5V
// SDA - pin 4
// SCL - pin 5


 Use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.

 This sketch demonstrates the use of the autoscroll()
 and noAutoscroll() functions to make new text scroll or not.

 The circuit:
 * LCD RS pin to digital pin     7
 * LCD Enable pin to digital pin 12
 * LCD D4 pin to digital pin     11
 * LCD D5 pin to digital pin     10
 * LCD D6 pin to digital pin     4
 * LCD D7 pin to digital pin     8
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

Русский вывод на МЭЛТ взят тут https://github.com/mk90/LiquidCrystalRus

OneWire DS18S20, DS18B20, DS1822 Temperature Example
http://ru.aliexpress.com/item/free-shipping-5pcs-DS18B20-To-92/32274668005.html

*/

// include the library code:
#include <LiquidCrystalRus.h>   // https://github.com/mk90/LiquidCrystalRus
#include <Wire.h>
#include "RTClib.h"
#include <OneWire.h>

#define BUZZER_PIN  3  // пин с пьезопищалкой
#define FREQ 1000      // противный писк

#define LED_RED    9    // красный светодиод
#define LED_GREEN  5    // зеленый светодиод
#define LED_BLUE   6    // синий светодиод

// таблица яркости RGB светодиода
byte L_red[12]=  {050,000,255, 00, 00, 00, 00, 00, 00, 00, 00,  00};
byte L_green[12]={000,000, 00, 00,050,000,255, 00, 00, 00, 00,  00};
byte L_blue[12]= {000,000, 00, 00, 00, 00, 00, 00,050, 00,255,  00};
int  cntrgb;

// initialize the library with the numbers of the interface pins
LiquidCrystalRus lcd(7, 12, 11, 10, 4, 8);
RTC_DS1307 RTC;  // часы реального времени

// температурный датчик 
OneWire  ds(2);  // on pin 2 (a 4.7K resistor is necessary)

int  CurT;  // текущая температура в 0.1 гр.(измерятся при новой минуте)

//String input; // строка даты
//String strtime; // строка времени

char ibuf[32];  // возьмем буфер с запасом для ввода
int  is=0;      // указатель в буфере
byte  old_min;   // старая минута
byte  old_sec;   // старая секунда

// адрес DS18B20
byte ds_addr[8];
byte show_t;     // показываем температуру

void setup() {
  //
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // Print a message to the LCD.
  //lcd.print("!!!");
  // инициализируем канал I2C
  Wire.begin();
  RTC.begin();
  // инициализируем последовательный порт
  Serial.begin(9600);
  //
  is=0;
  // режим пина с пищалкой
  pinMode(BUZZER_PIN, OUTPUT);
  //
  old_min=100;  // невозможная минута
  old_sec=100;  // невозможная секунда
  
  //
  pinMode(LED_RED,   OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE,  OUTPUT);
  cntrgb=0;
  //
  Serial.begin(9600);
  //
  // получим адрес температурногго датчика
  ds.reset_search();
  if ( !ds.search(ds_addr)) {
    Serial.println("No DS18B20 addresses.");
    ds_addr[0]=0;  // запишем в адрес "мусор",
    ds_addr[1]=1;  // чтобы не прошла проверка CRC
  }
  show_t=0;
  //
}

// измерить температуру в градусах Цельсия
float getTempC();

//
void loop() {
  //
  if (Serial.available() > 0) {
   char c;
    // read the oldest byte in the serial buffer:
    c= (char)Serial.read();
    if(is>30) is=0;  // сбросим строку
    ibuf[is]=c;
    is++;
    ibuf[is]=0;
    // YYYY:MM:DD:hh:mm:ss@ ?
    if (c=='@' && is>=20) {
      // установка часов
      int y,mo,d,h,m,s;
      //const char *str2=input.c_str();
      sscanf(ibuf,"%2d.%2d.%4d-%2d:%2d:%2d",&d,&mo,&y,&h,&m,&s);
      // Готовим для записи в RTC часы, минуты, секунды
      //Записываем эти данные во внутреннюю память часов.
      //С этого момента они начинают считать нужное для нас время
      //RTC.adjust(DateTime(y,mo,d,h,m,s));
      //
      Serial.print("We get: ");
      Serial.println(ibuf);
      Serial.print(y);
      Serial.print("-");
      Serial.print(mo);
      Serial.print("-");
      Serial.print(d);
      Serial.print(" ");
      Serial.print(h);
      Serial.print(":");
      Serial.print(m);
      Serial.print(":");
      Serial.println(s);
      is=0;
      lcd.clear();
      lcd.print("adjust time");
      delay(4000);
    } 
    
    delay(2);
  }else {
   char buf[40];  // возьмем буфер с запасом
   int  m,s,t1,t2;
    // ввода нет выводим часы
    // прочитаем часы
    DateTime now = RTC.now();
    //
    m=now.minute();  // текущая минута
    s=now.second();  // текущая секунда
    // 
    if(show_t) {
      // показываем температуру
      // curT измеряется при новой минуте
      t2=CurT%10;
      t1=(CurT-t2)/10;
      sprintf(buf,"Темпер. %3d.%1d C ",t1,t2);
    } else {
      // показываем дату
      sprintf(buf,"Дата: %02d.%02d.%4d", now.day(), now.month(), now.year());
    }
    //
    
    lcd.setCursor(0,0);
    lcd.print(buf);
       
    sprintf(buf,"Время: %02d:%02d:%02d", now.hour(), m, s);
    lcd.setCursor(0,1);
    lcd.print(buf);
    // разбереемся с минутами
    if ( m != old_min ) {
      // новая минута
      old_min=m;
      // эта новая минута час или полчаса? - каждые 30 минут - пищим
      if(m == 0 || m == 30 ) {
         tone(BUZZER_PIN, FREQ, 140);
      }
      //show_t=1-show_t;  // изменим режим отображения дата-температура
      // измерим и запомним температуру
      CurT=int(10*getTempC());    // 10 доли градуса
      //      
    }
    // каждая секунда - новый цвет 3-LED
    if (s != old_sec ) {
      old_sec=s;
      analogWrite(LED_RED,   L_red[cntrgb]);
      analogWrite(LED_GREEN, L_green[cntrgb]);
      analogWrite(LED_BLUE,  L_blue[cntrgb]);
      //    
      cntrgb=cntrgb+1;
      if(cntrgb>=12) cntrgb=0;
      //
      // изменим режим отображения дата-температура
      show_t=(s/15)%2;  // переключаем каждые 15 сек
    }
    delay(98);
    
  } 
  //
}


// функция считывания температуры с DS18B20
// измерить температуру в градусах Цельсия
float getTempC()
{
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  // byte addr[8];
  float celsius;
  
  
  // Serial.print("ROM =");
  // for( i = 0; i < 8; i++) {
  //  Serial.write(' ');
  //  Serial.print(addr[i], HEX);
  // }

  if (OneWire::crc8(ds_addr, 7) != ds_addr[7]) {
      Serial.println("CRC is not valid!");
      return -1000;
  }
  // Serial.println();
 
  // the first ROM byte indicates which chip
  switch (ds_addr[0]) {
    case 0x10:
      // Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      // Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      // Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return -1000;
  } 

  ds.reset();
  ds.select(ds_addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
  delay(800);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.
  
  present = ds.reset();
  ds.select(ds_addr);    
  ds.write(0xBE);         // Read Scratchpad

  // Serial.print("  Data = ");
  // Serial.print(present, HEX);
  // Serial.print(" ");
  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
    // Serial.print(data[i], HEX);
    // Serial.print(" ");
  }
  // Serial.print(" CRC=");
  // Serial.print(OneWire::crc8(data, 8), HEX);
  // Serial.println();

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
  celsius = (float)raw / 16.0;
//  fahrenheit = celsius * 1.8 + 32.0;

  return celsius;
}


